from setuptools import setup, find_packages

setup(name="ok_client_messenger",
      version="0.1",
      description="ok_client_messenger",
      author="Oleg Krechetov",
      author_email="kr.oleg@gmail.com",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )
